# Turn Theis into an easy to call function

Theis_Function <- function(origin_x, origin_y, Data_or_Location, Excel_SheetNumber_Name, Distance_from_OriginX, Distance_from_OriginY, Cell_Side_Length, Time_Days,Contour_Interval, Output_File_Directory, Output_File_Name, T_Overide, S_Overide, Q_Overide, mask) {
  require(tidyverse)
  require(data.table)
  require(directlabels)
  require(readxl)
  require(sp)
  require(raster)
  require(compiler)
  require(rgdal)
  
  # variable/input data ####
  if (is.character(Data_or_Location)) {
    Well_Data <- as.data.table(read_excel(path = paste0(Data_or_Location), sheet = Excel_SheetNumber_Name)) # input data
  }else{
    Well_Data <- Data_or_Location
  }
  
  # overide T, Q, and S value if constant
  if (missing(T_Overide)) {
    T_Overide = NULL
  }else{
    Well_Data$T <- T_Overide
  }
  
  if (missing(S_Overide)) {
    S_Overide = NULL
  }else{
    Well_Data$S <- S_Overide
  }
  
  if (missing(Q_Overide)) {
    Q_Overide = NULL
  }else{
    Well_Data$Q_gpm <- Q_Overide
  }
  
  if (missing(Distance_from_OriginX)) {
    distance_x = round(max(Well_Data$x) - min(Well_Data$x),0) * 1.25
  }else{
    distance_x = Distance_from_OriginX # how far out in x direction to go from origin
  }
  
  if (missing(Distance_from_OriginY)) {
    distance_y = round(max(Well_Data$y) - min(Well_Data$y),0) * 1.25
  }else{
    distance_y = Distance_from_OriginY # how far out in x direction to go from origin
  }
  
  if (missing(origin_x)) {
    origin_x = min(Well_Data$x) - (distance_x/10)
  }else{
    origin_x = origin_x
  }
  
  if (missing(origin_y)) {
    origin_y = min(Well_Data$y) - (distance_y/10)
  }else{
    origin_y = origin_y
  }
  
  # Check for mask value if not supplied set to NULL
  if (missing(mask)) {
    mask = NULL
  }
  
  cell_side = Cell_Side_Length # Just a single side of the cell, assume square cells
  
  Time = Time_Days  # in Days
  
  contour_interval = Contour_Interval # size of steps between contour intervals
  
  # Variable names to call in Well_Data file
  ## Well_ID - Name of Wells
  ## x - longitude location in projected coordinate system
  ## y - latitude location in projected coordinate system
  ## T - Transmissivity in gpd/ft
  ## Q_gpm - Pumping rate in gpm
  ## S - Storativity 
  ## Output_File_Directory - Directory were the file will be stored ex. "S:/Projects/Shapefiles", if running from current working directory can give relative path name ex. "Shapefiles"
  ## Output_File_Name - Name of file ex. "Sc_1_AvgT"
  
  # functions (crated or called from)
  uFun <- function(radius,Storativity,Transmissivity,time){
    ((radius^2)*Storativity)/(4*Transmissivity*time)#Calculates the u value in Theis equation
  }
  
  WuFun <-function(u){
    residual=1
    pow=2
    sign=-1
    Well_u = -0.5772-log(u)+u
    Well_Prev = Well_u
    while (residual > 1e-5){
      Well_u = Well_u + (((u^pow)/(pow*factorial(pow)))*sign)
      residual = abs(Well_Prev-Well_u)
      if(residual > 1000000){
        Well_u = 0
        break()
      }else{
        pow=pow+1
        sign=sign*-1
        Well_Prev=Well_u}
    }
    Well_u
  }
  
  drawdownFun <- function(Q,Transmissivity,Wu){
    dd = (Q/(4*pi*Transmissivity))*Wu
    return(dd)
  }
  
  euc.dist <- function(x1,x2,y1,y2){
    sqrt((x1 - x2)^2 + (y1 - y2)^2)
  }
  
  # compile functions for speed ####
  uFun <- cmpfun(uFun)
  WuFun <- cmpfun(WuFun)
  drawdownFun <- cmpfun(drawdownFun)
  euc.dist <- cmpfun(euc.dist)
  
  # Body of Code ####
  
  ## calculate the centriods for the grid
  data_grid = CJ(x = seq((origin_x + cell_side/2),((distance_x+ origin_x) - cell_side/2), cell_side), y = seq((origin_y + cell_side/2),((distance_y + origin_y) - cell_side/2), cell_side))
  ## create a cell ID to follow through analysis
  data_grid$Cell_ID <- c(1:length(data_grid$x))
  
  ## Calculate distance from wells to all grid centers
  Distances_to_grid <- lapply(Well_Data$Well_ID, function(j) data_grid[, euc.dist(x1 = Well_Data[Well_ID == j]$x, x2 = x,y1 = Well_Data[Well_ID ==j]$y, y2 = y)])
  
  ### Theis drawdown calculations
  value_u <- list()
  value_wu <- list()
  DD <- list()
  i = 1
  for(well in Well_Data$Well_ID){
    S = Well_Data[Well_ID == well]$S
    Tr = (Well_Data[Well_ID == well]$T/7.48)
    Q = (Well_Data[Well_ID == well]$Q_gpm*192.5)
    value_u[[i]] <- vapply(Distances_to_grid[[i]], function(x) uFun(x, S, Tr, Time), FUN.VALUE = as.double(1))
    value_wu[[i]] <- vapply(value_u[[i]], function(x) WuFun(x), as.double(1))
    DD[[i]] <- vapply(value_wu[[i]], function(x) drawdownFun(Q, Transmissivity = Tr, Wu = x), as.double(1))
    i = i + 1
  }
  
  
  ## Assign DD to grid
  data_grid <- as.data.frame(data_grid)
  k=1
  for(drawdown in DD){
    data_grid <- cbind(data_grid, drawdown)
    colnames(data_grid)[k+3] <- Well_Data$Well_ID[k]
    k=k+1
  }
  
  ## Calculate total drawdown of area
  data_grid$tot_drawdown <- apply(data_grid[,4:length(data_grid)], 1, sum)
  data_grid <- as.data.table(data_grid)
  
  # output to geotiff ####
  ## limit data to just total drawdown
  data_grid.export <- data_grid[,.(x,y, round(tot_drawdown,4))]
  ## give coordinates
  coordinates(data_grid.export) <- c('x','y')
  ## convert to standardized grid
  gridded(data_grid.export) <- T
  ## convert to raster
  data_grid.export <- raster(data_grid.export)
  ## mask raster to just outline of Nacatosch
  if (length(raster_mask) > 0){
    #data_grid.export <- setExtent(data_grid.export, mask, keepres=TRUE)
    data_grid.export <- mask(x = data_grid.export, mask = mask)
  }
  ## write to geotiff
  writeRaster(data_grid.export, paste0(Output_File_Directory,"/",Output_File_Name, ".tiff"), overwrite=T)
  
  ## write contours
  #number_Levels <- floor(cellStats(data_grid.export, max)/contour_interval)
  number_Levels = c(1,5,10,20,30,40,50,75,100,200,300,400,500,750,1000,1500,2000)
  contours <- rasterToContour(data_grid.export, levels = number_Levels)
  writeOGR(contours, dsn = Output_File_Directory, layer = paste0(Output_File_Name,"_contours"), driver = "ESRI Shapefile", overwrite_layer = T)
  
  # Output
  plot2 <- ggplot() + geom_raster(data = data_grid, aes(x = x, y = y, fill = tot_drawdown)) + geom_contour(data = data_grid, aes(x = x, y = y, z = tot_drawdown, color = ..level..), color = 'white', binwidth = 50) + geom_point(data = Well_Data, aes(x = x, y = y), color = 'red') + scale_color_continuous(guide = F) + scale_fill_continuous(name = "Drawdown")
  
  return(plot2)
}
