import arcpy
from arcpy import env
from arcpy.sa import *

arcpy.CheckOutExtension("Spatial")
env.workspace = '..\\geodatabase\Nacatoch_Interpolation_Work.gdb'
env.overwriteOutput = True

# Raster datasets for caclulations
TDS = Raster("TDS_IDW")
Nac_Top = Raster("Top_Nac_EBK")
Nac_Bot = Raster("Bot_Nac_EBK")
WL = Raster("WL_Depths_IDW")
PS = Raster("PS_IDW")
Aquifer_Designation = Raster("Aquifer_Designation") # outcrop and subcrop indicator raster
Nac_Storativity = Raster("S_IDW")
Nac_Specific_Yield = Raster("Sy_IDW")
Nac_Transmissivity = Raster("T_gpdft_IDW")

# Constants
Area = 1320 * 1320 # Is the area of each cell 1,320 feet per side or quarter-mile

# Reclassify TDS interpolation to create salinity zones
Zones = Reclassify(in_raster= TDS,reclass_field="VALUE",remap=RemapRange([[0,1000,1],[1000.000001,3000,2],[3000.000001,10000,3],[10000.00001,35000,4],[35000.000001,100000,5]]))
Zones.save("Salinity_Zones")

# Create Outcrop and Subcrop zones by setting raster zones to 1 or 0, will then add later to create a complete volume raster
Unconfined_Area = Con(Aquifer_Designation == 2,0,1)
Confined_Area = Con(Aquifer_Designation == 1, 0, 1)

# Correct for negative depth values in top of nac layer by turning all values to a depth of 0 or at the surface.
Nac_Top2 = Con(Nac_Top > 0, Nac_Top, 0)

#  Calculate Volumes
Nac_Thickness = Nac_Bot - Nac_Top
# Correct thickness to 5 feet if values are negative
Nac_Thickness = Con(Nac_Thickness < 0, 5,  Nac_Thickness)
Nac_Thickness.save("Nac_Thickness")

Vol_Unconfined = ((Area * Nac_Specific_Yield * (Nac_Bot - WL)) * Unconfined_Area) * 2.29569e-5 # Calculatin volume of water in the unconfined areas and converting to ac-ft
Vol_Confined = ((Area * Nac_Specific_Yield * (Nac_Top - WL)) * Confined_Area) * 2.29569e-5

# Get volume in sand formations by multilpying by the percent sand
Vol_Unconfined = Vol_Unconfined * PS
Vol_Confined = Vol_Confined * PS

# Save rasters of volume
Vol_Unconfined.save("Volume_Unconfined")
Vol_Confined.save("Volume_Confined")

# Calculate volumes per salinity zones
Vol_Zones_Uncon = ZonalStatisticsAsTable(in_zone_data=Zones,zone_field="VALUE",in_value_raster =Vol_Unconfined,out_table="Vol_Zones_Unconfined",statistics_type="SUM")
Vol_Zones_Con = ZonalStatisticsAsTable(in_zone_data=Zones,zone_field="VALUE",in_value_raster =Vol_Confined,out_table="Vol_Zones_Confined",statistics_type="SUM")

Vol_Combined = Vol_Confined + Vol_Unconfined
Vol_Combined.save("Vol_All")
# Extract top, bot, thick, and percent sand layers by TDS zones
# Turn each salinity zone to a value of one and mulipty by Nact top,bot,thick,ps
fr_zone = Con(Zones != 1, 0, 1)
fr_zone = SetNull(fr_zone, 1, "VALUE = 0")
fr_top = Nac_Top * fr_zone
fr_bot = Nac_Bot * fr_zone
fr_thk = Nac_Thickness * fr_zone
fr_ps = PS * fr_zone
fr_vol = Vol_Combined * fr_zone

fr_top.save("fr_top")
fr_bot.save("fr_bot")
fr_thk.save("fr_thk")
fr_ps.save("fr_ps")
fr_vol.save("fr_vol")

ss_zone = Con(Zones != 2, 0, 1)
ss_zone = SetNull(ss_zone, 1, "VALUE = 0")
ss_top = Nac_Top * ss_zone
ss_bot = Nac_Bot * ss_zone
ss_thk = Nac_Thickness * ss_zone
ss_ps = PS * ss_zone
ss_vol = Vol_Combined * ss_zone

ss_top.save("ss_top")
ss_bot.save("ss_bot")
ss_thk.save("ss_thk")
ss_ps.save("ss_ps")
ss_vol.save("ss_vol")

ms_zone = Con(Zones != 3, 0, 1)
ms_zone = SetNull(ms_zone, 1, "VALUE = 0")
ms_top = Nac_Top * ms_zone
ms_bot = Nac_Bot * ms_zone
ms_thk = Nac_Thickness * ms_zone
ms_ps = PS * ms_zone
ms_vol = Vol_Combined * ms_zone

ms_top.save("ms_top")
ms_bot.save("ms_bot")
ms_thk.save("ms_thk")
ms_ps.save("ms_ps")
ms_vol.save("ms_vol")

vs_zone = Con(Zones != 4, 0, 1)
vs_zone = SetNull(vs_zone, 1, "VALUE = 0")
vs_top = Nac_Top * vs_zone
vs_bot = Nac_Bot * vs_zone
vs_thk = Nac_Thickness * vs_zone
vs_ps = PS * vs_zone
vs_vol = Vol_Combined * vs_zone

vs_top.save("vs_top")
vs_bot.save("vs_bot")
vs_thk.save("vs_thk")
vs_ps.save("vs_ps")
vs_vol.save("vs_vol")

arcpy.CheckInExtension("Spatial")